#define SerialDevice SerialUSB
#define LED_PIN 10
#include "FastLED.h"
#define NUM_LEDS 19
CRGBArray<NUM_LEDS> leds;

#include "Keyboard.h"




enum {
  LedGs8Bit = 0x31,//49
  LedGs8BitMulti = 0x32,//50
  LedGs8BitMultiFade = 0x33,//51
  LedFet = 0x39,//57
  LedGsUpdate = 0x3C,//60
  LedDirect = 0x82,
};

typedef union {
  uint8_t base[64];
  struct {
    struct {
      uint8_t dstNodeID;
      uint8_t srcNodeID;
      uint8_t length;
      uint8_t cmd;
    };
    union {
      struct { //39
        uint8_t color[11][3];//CRGB
      };
      struct { //9
        uint8_t BodyLed;
        uint8_t ExtLed;
        uint8_t SideLed;
      };
      struct { //LedGs8Bit
        uint8_t index;
        uint8_t r;
        uint8_t g;
        uint8_t b;
      };
      struct { //LedGs8BitMulti,LedGs8BitMultiFade
        uint8_t start;
        uint8_t end;//length
        uint8_t skip;
        uint8_t mr;
        uint8_t mg;
        uint8_t mb;
        uint8_t speed;
      };
    };
  };
} Packet;

static Packet req;

static uint8_t len, r, checksum;
static bool escape = false;

static uint8_t packet_read() {
  while (SerialDevice.available()) {
    r = SerialDevice.read();
    if (r == 0xE0) {
      len = 0;
      checksum = 0;
      continue;
    }
    if (r == 0xD0) {
      escape = true;
      continue;
    }
    if (escape) {
      r++;
      escape = false;
    }

    if (len - 3 == req.length && checksum == r) {
      return req.cmd;
    }
    req.base[len++] = r;
    checksum += r;
  }
  return 0;
}

void setup() {
  FastLED.addLeds<WS2812B, LED_PIN, GRB>(leds, NUM_LEDS);
  FastLED.setBrightness(100);
  FastLED.clear();
  SerialDevice.begin(115200);
  leds(0, 7) = 0xFFFFFF;
  FastLED.delay(1000);
  Keyboard.begin();
  for(int i=2;i<=9;i++){
    pinMode(i,INPUT_PULLUP);
    if (digitalRead(i) == HIGH){
      pinMode(i,OUTPUT);
      digitalWrite(i,LOW);
    pinMode(14,INPUT);
    pinMode(15,INPUT);
    pinMode(16,INPUT);
    }
  }
  

}

unsigned long fade_start, fade_end, progress;
uint8_t led_start, led_end, fade_tag;
CRGB fade_prev, fade_taget;

void loop() {
    

  
  switch (packet_read()) {
    case LedGs8Bit:
      leds[(req.index+1)*2-1] = CRGB(req.r, req.g, req.b);
      leds[(req.index+1)*2-2] = CRGB(req.r, req.g, req.b);
      break;

    case LedGs8BitMulti:
      if (req.end == 0x20) {
        req.end = NUM_LEDS;
      }
      leds((req.start)*2, (req.end)*2) = CRGB(req.mr, req.mg, req.mb);
      fade_prev = CRGB(req.mr, req.mg, req.mb);
      fade_tag = 0;
      break;

    case LedGs8BitMultiFade:
      fade_taget = CRGB(req.mr, req.mg, req.mb);
      fade_start = millis();
      fade_end = fade_start + (4095 / req.speed * 8);
      led_start = (req.start)*4;
      led_end = (req.end - 1)*4;
      fade_tag = 1;
      break;

    case LedFet://框体灯，只有白色，值代表亮度，会多次发送实现渐变，需要立刻刷新
      leds[17] = blend(0x000000, 0xFFFFFF, req.BodyLed);
      leds[18] = blend(0x000000, 0xFFFFFF, req.ExtLed);//same as BodyLed
      leds[19] = blend(0x000000, 0xFFFFFF, req.SideLed);//00 or FF
      FastLED.show();
      break;

    case LedGsUpdate://提交灯光数据
      FastLED.show();
      break;
  }


  if (digitalRead(2) == HIGH ){
    Keyboard.press(119);
  }else{
    Keyboard.release(119);
  }  

  if (digitalRead(3) == HIGH ){
    Keyboard.press(101);
  }else{
    Keyboard.release(101);
  }  

  if (digitalRead(4) == HIGH ){
    Keyboard.press(100);
  }else{
    Keyboard.release(100);
  }  

  if (digitalRead(5) == HIGH ){
    Keyboard.press(99);
  }else{
    Keyboard.release(99);
  }  

  if (digitalRead(6) == HIGH ){
    Keyboard.press(120);
  }else{
    Keyboard.release(120);
  }  

  if (digitalRead(7) == HIGH ){
    Keyboard.press(122);
  }else{
    Keyboard.release(122);
  }  

  if (digitalRead(8) == HIGH ){
    Keyboard.press(97);
  }else{
    Keyboard.release(97);
  }  

  if (digitalRead(9) == HIGH ){
    Keyboard.press(113);
  }else{
    Keyboard.release(113);
  }  

//  if (digitalRead(16) == HIGH ){
//    Keyboard.press(51);
//  }else{
//    Keyboard.release(51);
//  }  
//
//  if (digitalRead(14) == HIGH ){
//    Keyboard.press(48);
//  }else{
//    Keyboard.release(48);
//  }  
//
//
//  if (digitalRead(15) == HIGH ){
//    Keyboard.press(57);
//  }else{
//    Keyboard.release(57);
//  }  

  if (!fade_tag)return;
  if (millis() > fade_end) {
    progress = 255;
    fade_tag = 0;
  } else {
    progress = map(millis(), fade_start, fade_end, 0, 255);
  }
  leds(led_start, led_end) = blend(fade_prev, fade_taget, progress);
  FastLED.show();




}
